@@ -26,32 +26,14 @@ module ActiveRecord
     end
 
     def self.run
-      pools = []
-
-      if ActiveRecord::Base.legacy_connection_handling
-        ActiveRecord::Base.connection_handlers.each do |key, handler|
-          pools.concat(handler.connection_pool_list.reject { |p| p.query_cache_enabled }.each { |p| p.enable_query_cache! })
-        end
-      else
-        pools.concat(ActiveRecord::Base.connection_handler.all_connection_pools.reject { |p| p.query_cache_enabled }.each { |p| p.enable_query_cache! })
-      end
-
-      pools
+      ActiveRecord::Base.connection_handler.each_connection_pool.reject { |p| p.query_cache_enabled }.each { |p| p.enable_query_cache! }
     end
 
     def self.complete(pools)
-      pools.each { |pool| pool.disable_query_cache! }
+      pools.each { |pool| pool.disable_query_cache! unless pool.discarded? }
 
-      if ActiveRecord::Base.legacy_connection_handling
-        ActiveRecord::Base.connection_handlers.each do |_, handler|
-          handler.connection_pool_list.each do |pool|
-            pool.release_connection if pool.active_connection? && !pool.connection.transaction_open?
-          end
-        end
-      else
-        ActiveRecord::Base.connection_handler.all_connection_pools.each do |pool|
-          pool.release_connection if pool.active_connection? && !pool.connection.transaction_open?
-        end
+      ActiveRecord::Base.connection_handler.each_connection_pool do |pool|
+        pool.release_connection if pool.active_connection? && !pool.connection.transaction_open?
       end
     end
 
